@@ -27,6 +27,7 @@
 #include <dirent.h>
 #include <sys/types.h>
 #include <sys/wait.h>
+#include <math.h>
 
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
@@ -60,6 +61,7 @@
 #define FIRE_CPU	1
 #define FIRE_MEM	2
 #define FIRE_NET	3
+#define FIRE_FILE	4
 
 #define NET_SPD_PPP	56
 #define NET_SPD_ETH	100
@@ -82,6 +84,7 @@ typedef struct {
 
 	int x;			/* Window X position */
 	int y;			/* Window Y position */
+	int sticky;		/* Window sticky */
 
 	unsigned char *flame;
 	unsigned char cmap[CMAPSIZE];
@@ -100,6 +103,7 @@ typedef struct {
 int update_cpu();
 int update_mem();
 int update_net();
+int update_file();
 int change_cpu(int);
 void change_flame(int);
 GdkCursor *setup_cursor();
@@ -131,6 +135,9 @@ int cpu_id = 0;
 int cpu_nice = 1;
 char net_dev[16] = "ppp0";
 int net_spd = 0;
+char *file_name = NULL;
+int file_max = 100;
+int file_min = 0;
 int cmap = 0;
 int lock = 0;
 int proximity = 0;
@@ -190,9 +197,11 @@ main(int argc, char **argv)
 						cpu_nice = !cpu_nice;
 					else if (event->button.button == 3)
 						change_flame(0);
-					else if (event->button.button == 4)
+					break;
+				case GDK_SCROLL:
+					if (event->scroll.direction == GDK_SCROLL_UP)
 						next = 1;
-					else if (event->button.button == 5)
+					else if (event->scroll.direction == GDK_SCROLL_DOWN)
 						next = 1;
 					break;
 				case GDK_ENTER_NOTIFY:
@@ -220,6 +229,11 @@ main(int argc, char **argv)
 				else if (monitor == FIRE_MEM)
 					monitor = FIRE_NET;
 				else if (monitor == FIRE_NET)
+					if (file_name)
+						monitor = FIRE_FILE;
+					else
+						monitor = FIRE_CPU;
+				else if (monitor == FIRE_FILE)
 					monitor = FIRE_CPU;
 			}
 		}
@@ -232,6 +246,8 @@ main(int argc, char **argv)
 				load = update_mem();
 			else if (monitor == FIRE_NET)
 				load = update_net();
+			else if (monitor == FIRE_FILE)
+				load = update_file();
 
 			if (load > 100)
 				load = 100;
@@ -324,7 +340,7 @@ int
 update_net()
 {
 	glibtop_netload netload;
-	static unsigned long oldtotal = 0;
+	static guint64 oldtotal = 0;
 	int percent;
 
 	glibtop_get_netload(&netload,net_dev);
@@ -337,6 +353,32 @@ update_net()
 	return percent;
 }
 
+/******************************************/
+/* Update file statistics                 */
+/******************************************/
+
+int
+update_file()
+{
+	char buf[128];
+	float percent, number;
+	FILE *fp;
+
+	if (!(fp = fopen(file_name, "r")))
+		return 100;
+
+	/* First number only. Complex parsing should be done in */
+	/* external program and value saved to monitored file.  */
+
+	fgets(buf, sizeof (buf), fp);
+	number = atof(buf);
+	fclose(fp);
+
+	percent = 100 * (number - file_min) / (file_max - file_min);
+
+	return (int) percent;
+}
+
 /******************************************/
 /* Change CPU monitor                     */
 /******************************************/
@@ -432,6 +474,7 @@ inline void
 draw_fire(unsigned int load)
 {
 	int x, y, i, j;
+	double psi;
 
 	/* Setup hot spots */
 	for (i = 0; i < ((load >> 3) + 2); i++)
@@ -465,16 +508,25 @@ draw_fire(unsigned int load)
 				}
 			} else if (monitor == FIRE_MEM) {
 				/* Grid effect for memory arrays */
-				for (i = 0; i< 4; i++) {
+				for (i = 0; i < 4; i++) {
 					for (j = 0; j < 4; j++)
 						burn_spot(13+i*10, 13+j*10, 3);
 				}
 			} else if (monitor == FIRE_NET) {
 				/* Marching ants for network traffic */
-				for (i = 7; i< XMAX-10;i+=6) {
+				for (i = 7; i < XMAX-10; i+=6) {
 					j = i + (proximity/12) % 6;
 					burn_spot(j, 27, 3);
 				}
+			} else if (monitor == FIRE_FILE) {
+				/* Rotating disk platter */
+				burn_spot(26, 26, 3);
+				for (i = (proximity/4) % 4; i < 40; i+=4) {
+					psi = i * 3.14 / 20.0;
+					x = floor(sin(psi) * 18) + 26;
+					y = floor(-cos(psi) * 8) + 26;
+					burn_spot(x, y, 3);
+				}
 			}
 		}
 	}
@@ -556,7 +608,6 @@ make_wmfire_dockapp(void)
 	wmhints.icon_y = 0;
 	wmhints.window_group = win;
 	wmhints.flags = StateHint | IconWindowHint | IconPositionHint | WindowGroupHint;
-	XSetWMHints(GDK_WINDOW_XDISPLAY(bm.win), win, &wmhints);
 
 	bm.gc = gdk_gc_new(bm.win);
 
@@ -579,8 +630,13 @@ make_wmfire_dockapp(void)
 
 	gdk_window_show(bm.win);
 
+	/* Moved after gdk_window_show due to change in GTK 2.4 */
+	XSetWMHints(GDK_WINDOW_XDISPLAY(bm.win), win, &wmhints);
+
 	if (bm.x > 0 || bm.y > 0)
 		gdk_window_move(bm.win, bm.x, bm.y);
+	if (bm.sticky)
+		gdk_window_stick(bm.win);
 #undef MASK
 }
 
@@ -597,7 +653,7 @@ read_config(int argc, char **argv)
 	bm.flame = fire[cmap].data;
 
 	/* Parse command options */
-	while ((i = getopt(argc, argv, "c:mni:s:xpf:lbhg:S:")) != -1) {
+	while ((i = getopt(argc, argv, "c:mni:s:xF:H:L:pf:lbhg:yS:")) != -1) {
 		switch (i) {
 		case 'S':
 			if (optarg)
@@ -613,6 +669,9 @@ read_config(int argc, char **argv)
 					bm.y = gdk_screen_height() - 64 + bm.y;
 			}
 			break;
+		case 'y':
+			bm.sticky = 1;
+			break;
 		case 'c':
 			monitor = FIRE_CPU;
 			if (optarg)
@@ -626,7 +685,7 @@ read_config(int argc, char **argv)
 			break;
 		case 'i':
 			if (optarg)
-				strcpy(net_dev, optarg);
+				strncpy(net_dev, optarg, sizeof (net_dev));
 			break;
 		case 's':
 			if (optarg)
@@ -639,6 +698,19 @@ read_config(int argc, char **argv)
 		case 'x':
 			cpu_nice = 0;
 			break;
+		case 'F':
+			monitor = FIRE_FILE;
+			if (optarg)
+				file_name = strdup(optarg);
+			break;
+		case 'H':
+			if (optarg)
+				file_max = atof(optarg);
+			break;
+		case 'L':
+			if (optarg)
+				file_min = atof(optarg);
+			break;
 		case 'p':
 			monitor = FIRE_NONE;
 			break;
@@ -679,12 +751,16 @@ do_help(void)
 	fprintf(stderr, "\nWmfire - Flaming Monitor Dock V %s\n\n", VERSION);
 	fprintf(stderr, "Usage: wmfire [ options... ]\n\n");
 	fprintf(stderr, "\t-g [{+-}X{+-}Y]\t\tinital window position\n");
+	fprintf(stderr, "\t-y\t\t\tset window sticky\n");
 	fprintf(stderr, "\t-c [0..%d]\t\tmonitor smp cpu X\n", GLIBTOP_NCPU-1);
 	fprintf(stderr, "\t-m\t\t\tmonitor memory load\n");
 	fprintf(stderr, "\t-n\t\t\tmonitor network load\n");
+	fprintf(stderr, "\t-F [...]\t\tmonitor file\n");
 	fprintf(stderr, "\t-i [...]\t\tchange network interface (default:%s)\n", net_dev);
 	fprintf(stderr, "\t-s [...]\t\tchange network speed (ppp:%dK) (eth:%dM)\n", NET_SPD_PPP, NET_SPD_ETH);
 	fprintf(stderr, "\t-x\t\t\texclude nice'd cpu load\n");
+	fprintf(stderr, "\t-H [...]\t\tset file maximum (high) value\n");
+	fprintf(stderr, "\t-L [...]\t\tset file minumum (low) value\n");
 	fprintf(stderr, "\t-p\t\t\tfire effect only\n");
 	fprintf(stderr, "\t-f [1..%d]\t\tchange flame colour\n\t\t\t\t", NFLAMES);
 	for (i = 0; i < NFLAMES; i++)
